@@ -166,7 +166,7 @@ int get_mmc_profile (int fd)
   unsigned char buf[8],inq[128];
   unsigned char *sense=cmd.sense();
   int i;
-  int profile=0,once=1,blank=0,err;
+  int profile=0,once=1,blank=0,err,erasable=0;
   unsigned int len;
 
   /* For valgrind */
@@ -200,12 +200,13 @@ int get_mmc_profile (int fd)
     cmd[9] = 0;
     if ((err=cmd.transport(READ,buf,sizeof(buf)))) {
       /*sperror ("GET CONFIGURATION",err);*/
-      fprintf (stderr, "Could not determine drive profile: non-MMC unit?\n");
-      goto bail;
+      /* this is not a fatal error -- some older drives support MMC-1
+       * but don't support the GET CONFIGURATION command (which is part
+       * of the MMC-2 spec). */
+    } else {
+        if ((profile = buf[6]<<8|buf[7]) || !once) break;
     }
 
-    if ((profile = buf[6]<<8|buf[7]) || !once) break;
-
     // no media?
     cmd[0] = 0;	// TEST UNIT READY
     cmd[5] = 0;
@@ -236,6 +237,24 @@ int get_mmc_profile (int fd)
   // see if it's blank media
   if ((disc_info[2]&3) == 0)	blank=0x10000;
 
+  if (!profile ) {
+      /* if the profile has not yet been set, we're dealing with an older
+       * CD-R or CD-RW burner (which doesn't know the GET CONFIGURATION
+       * command.  Do some digging into the disc type to figure out what's
+       * in the drive */
+      erasable = ((disc_info[2] & 16));
+      if (blank && !erasable) {
+          profile = 0x09;  /* CD-R                        */
+      } else if (erasable) {
+          profile = 0x0a;  /* CD-RW                       */
+      } else if (disc_info[8] == 0x00) {
+          profile = 0x08;  /* Commercial CDs and Audio CD */
+      } else {
+         fprintf (stderr, "Could not determine profile or type of media\n");
+         goto bail;
+      }
+  }
+
   if (profile != 0x1A && profile != 0x13 && profile != 0x12)
     return blank|profile;
 
@@ -500,20 +519,12 @@ retry:
 		}
 	    }
 
-	    nwa = 0;
-	    if (buf[7]&1)	// NWA_V
-	    {	nwa  = buf[12]<<24;
-		nwa |= buf[13]<<16;
-		nwa |= buf[14]<<8;
-		nwa |= buf[15];
-	    }
-	    free_blocks  = buf[16]<<24;
-	    free_blocks |= buf[17]<<16;
-	    free_blocks |= buf[18]<<8;
-	    free_blocks |= buf[19];
-	    ret = nwa + free_blocks;
-	    break;
+	    ret = buf[24]<<24;
+	    ret |= buf[25]<<16;
+	    ret |= buf[26]<<8;
+	    ret |= buf[27];
 
+	    break;
 	default:
 	    break;
     }
