@@ -58,14 +58,6 @@ enum {
 	CANCEL_CD_RECORD,
 };
 
-static GladeXML *xml;
-
-static NautilusBurnRecorder *recorder = NULL;
-static int                   cancel   = CANCEL_NONE;
-/* For the image spinning */
-static GList                *image_list;
-static GList                *files_for_cleanup = NULL;
-
 typedef struct {
 	char              *source_device;
 	char              *source_iso;
@@ -83,10 +75,19 @@ typedef struct {
 	gint64                         size;
 } NautilusBurnerWriteData;
 
+static GladeXML *xml;
+
+static NautilusBurnRecorder    *recorder = NULL;
+static int                      cancel   = CANCEL_NONE;
+/* For the image spinning */
+static GList                   *image_list;
+static GList                   *files_for_cleanup = NULL;
+static NautilusBurnerWriteData *write_data = NULL;
+
 static int write_disc (NautilusBurnerWriteData *data);
 
 static void
-nautilus_burner_write_data_free (NautilusBurnerWriteData *data)
+nautilus_burn_progress_write_data_free (NautilusBurnerWriteData *data)
 {
 	if (! data)
 		return;
@@ -350,6 +351,45 @@ nautilus_burn_progress_temp_files_cleanup (void)
 	}
 }
 
+static void
+nautilus_burn_progress_set_write_data (NautilusBurnerWriteData *data)
+{
+	/* just set a global for now */
+	write_data = data;
+}
+
+static NautilusBurnerWriteData *
+nautilus_burn_progress_get_write_data (void)
+{
+	return write_data;
+}
+
+static void
+nautilus_burn_progress_cleanup (void)
+{
+	NautilusBurnerWriteData *data;
+
+	nautilus_burn_progress_image_cleanup ();
+	nautilus_burn_progress_temp_files_cleanup ();
+
+	data = nautilus_burn_progress_get_write_data ();
+	if (data)
+		nautilus_burn_progress_write_data_free (data);
+}
+
+static int
+nautilus_burn_progress_write_disc (void)
+{
+	NautilusBurnerWriteData *data;
+	int                      res;
+
+	data = nautilus_burn_progress_get_write_data ();
+
+	res = write_disc (data);
+
+	return res;
+}
+
 static void
 animation_changed_cb (NautilusBurnRecorder *recorder,
 		      gboolean              spinning,
@@ -451,6 +491,9 @@ drive_type_get_supported_media (int type)
 	if (type & NAUTILUS_BURN_DRIVE_TYPE_DVD_PLUS_R_RECORDER) {
 		string = add_desc (string, "DVD+R");
 	}
+	if (type & NAUTILUS_BURN_DRIVE_TYPE_DVD_PLUS_R_DL_RECORDER) {
+		string = add_desc (string, "DVD+R DL");
+	}
 	if (type & NAUTILUS_BURN_DRIVE_TYPE_DVD_PLUS_RW_RECORDER) {
 		string = add_desc (string, "DVD+RW");
 	}
@@ -805,29 +848,30 @@ setup_cancel_button (void)
 }
 
 static void
-write_another_disc_cb (GtkWidget               *widget,
-		       NautilusBurnerWriteData *data)
+write_another_disc_cb (GtkWidget *widget,
+		       gpointer   user_data)
 {
-	int res;
+	int                      res;
+	NautilusBurnerWriteData *data;
 
 	setup_cancel_button ();
 
+	data = nautilus_burn_progress_get_write_data ();
 	nautilus_burn_drive_eject ((NautilusBurnDrive *)data->drive);
 
-	res = write_disc (data);
+	res = nautilus_burn_progress_write_disc ();
 
 	if (res == NAUTILUS_BURN_RECORDER_RESULT_CANCEL) {
 		gtk_main_quit ();
 	}
-
-	nautilus_burner_write_data_free (data);
 }
 
 static void
-setup_close_button (NautilusBurnerWriteData *data)
+setup_close_button (void)
 {
-	GtkWidget *close_button;
-	GtkWidget *cancel_button;
+	GtkWidget               *close_button;
+	GtkWidget               *cancel_button;
+	NautilusBurnerWriteData *data;
 
 	cancel_button = glade_xml_get_widget (xml, "cancel_button");
 	close_button = glade_xml_get_widget (xml, "close_button");
@@ -836,14 +880,17 @@ setup_close_button (NautilusBurnerWriteData *data)
 	gtk_widget_show (close_button);
 	gtk_widget_grab_focus (close_button);
 
+	g_signal_handlers_disconnect_by_func (close_button, gtk_main_quit, NULL);
 	g_signal_connect (close_button, "clicked", G_CALLBACK (gtk_main_quit), NULL);
 
+	data = nautilus_burn_progress_get_write_data ();
 	if (data) {
 		GtkWidget *write_another_button;
 		write_another_button = glade_xml_get_widget (xml, "write_another_button");
 		gtk_widget_show (write_another_button);
 
-		g_signal_connect (write_another_button, "clicked", G_CALLBACK (write_another_disc_cb), data);
+		g_signal_handlers_disconnect_by_func (write_another_button, write_another_disc_cb, NULL);
+		g_signal_connect (write_another_button, "clicked", G_CALLBACK (write_another_disc_cb), NULL);
 	}
 }
 
@@ -1144,12 +1191,12 @@ write_disc (NautilusBurnerWriteData *data)
 	if (res == NAUTILUS_BURN_RECORDER_RESULT_FINISHED) {
 		nautilus_burn_progress_set_text (_("Complete"));
 
-		setup_close_button (data);
+		setup_close_button ();
 
 	} else if (res != NAUTILUS_BURN_RECORDER_RESULT_CANCEL) {
 		nautilus_burn_progress_set_text (_("An error occurred while writing"));
 
-		setup_close_button (data);
+		setup_close_button ();
 
 		show_error_message (recorder, error);
 		g_error_free (error);
@@ -1344,7 +1391,7 @@ burn_cd (const NautilusBurnDrive *drive,
 		nautilus_burn_progress_set_time (-1);
 		g_free (str);
 
-		setup_close_button (NULL);
+		setup_close_button ();
 
 		res = NAUTILUS_BURN_RECORDER_RESULT_FINISHED;
 		goto out;
@@ -1399,7 +1446,8 @@ burn_cd (const NautilusBurnDrive *drive,
 		data->speed  = speed;
 		data->size   = burn_size;
 
-		res = write_disc (data);
+		nautilus_burn_progress_set_write_data (data);
+		res = nautilus_burn_progress_write_disc ();
 
 		if (source_iso == NULL) {
 			nautilus_burn_progress_temp_files_add (filename);
@@ -1979,8 +2027,6 @@ selection_dialog_response (GtkWidget                *dialog,
 			       overburn,
 			       burnproof);
 
-		nautilus_burn_progress_image_cleanup ();
-
 		if (res == NAUTILUS_BURN_RECORDER_RESULT_CANCEL) {
 			gtk_main_quit ();
 		}
@@ -2170,7 +2216,7 @@ main (int argc, char *argv[])
 	g_free (burn_data->source_device);
 	g_free (burn_data);
 
-	nautilus_burn_progress_temp_files_cleanup ();
+	nautilus_burn_progress_cleanup ();
 
 	return 0;
 }
